/*
 * application.C
 *
 * The main 'global' body of the application.
 *
 * Vincent Mallet, mars 1997, 1999
 */

// $Id: application.C,v 1.3 1999/04/20 23:09:49 vmallet Exp $

/* ViMmail - a small mail reader
 * Copyright (C) 1997, 1999 Vincent Mallet - vmallet@enst.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <Xm/Xm.h>
#include <Xm/List.h>

#include <stdlib.h>

#include "VimDebug.H"

#include "application.H"
#include "message_file.H"
#include "mailarea.H"
#include "filelist.H"
#include "config.H"
#include "configmotif.H"


// include "textarea.H"
// include "scale.H"


#define INVALID_CURSOR  -1

// Application creator
void Application:: make(int argc, char **argv)
{
  interface.make(argc, argv);
  config.make(interface.toplevel);
  _cursor  = INVALID_CURSOR;
  nb_files = 0;
}


// Start the application
void Application:: start_app(void)
{
  config.getconfig();
  config.save_config();

  new_message_file(config.mail_file);
  new_message_file(config.mbox_file);

  check_files(); // a l'heure qu'il est, c'est con ca.. vu qu'on vient de les lire! (les fichiers)
  set_checkfiles_timer();

  set_active_message_file(0);

  interface.run_till_death();
}


// terminate the application (clean up)
void Application:: end_app(void)
{
  for (int i = 0; i < nb_files; i++)
    if (mbox[i])
      delete mbox[i];

  exit(0);
}


// Open a not-yet-opened mail file. This file becomes
// the active mail file.
void Application:: new_message_file(char *name)
{
  int index;

  if (nb_files < VM_MAXFILES) {

    index = nb_files;

    mbox[index] = new Message_File(name);

    interface.filelist.add_entry(name);
    //    interface->scale->resize_object();

    if (config.show_last_message)
      mbox[index]->end();

    nb_files++;

    set_active_message_file(index);
  }
}


// rescan an already loaded mail file
void Application:: rescan_message_file(int index) // default index=CURRENT
{
  if (index == CURRENT) {
    index = _cursor;
    printf("foufafsfd\n");
  }
  printf("index == %d\n", index);
  mbox[index]->rescan_messages();
  if (config.show_last_message)
    mbox[index]->end();
  if (index == _cursor)
    set_active_message_file(index);
}


// rescan all already loaded mail files
void Application:: rescan_all_message_files(void)
{
  for (int i = 0; i < nb_files; i++)
    rescan_message_file(i);
}


// activate the specified mail file
void Application:: set_active_message_file(int index)
{
  int save_pos;

  //  if (index == _cursor)
  //    return;

  _cursor = index;

  interface.filelist.set_current(index);

  // met a jour la liste des messages
  interface.msglist.clear();

  save_pos = mbox[_cursor]->cursor();
  for (mbox[_cursor]->start(); !mbox[_cursor]->off(); mbox[_cursor]->forth()) {
    
    char *tmp = mbox[_cursor]->item().header_toString();

    interface.msglist.add_entry(tmp);

    free(tmp); // c'est beau c++
  }

  mbox[_cursor]->set_current(save_pos);

  if (!mbox[_cursor]->empty())
    set_active_message(mbox[_cursor]->cursor());
  else
    clear_message_window();
}


// activate(select) a message
void Application:: set_active_message(int index)
{
  //  if (index == mbox[_cursor]->cursor())
  //  return;
  char *tmp;

  mbox[_cursor]->set_current(index);
  interface.msglist.set_current(index);
  tmp = mbox[_cursor]->item().core_toString();
  //  interface->textarea->set_text(tmp);
  interface.mailarea.set_text(tmp);
  free(tmp);
}



// clean up the message window
void Application:: clear_message_window(void)
{
  //  interface->textarea->set_text("");
  interface.mailarea.set_text("");
}


// reconfigure certains elements (reponse a un changement
// de config par menu options par exemple)
void Application:: reconfigure(void)
{
  interface.reconfigure();
}


// save a message
void Application:: save_message(int index) // default index=CURRENT
{
  if (index == CURRENT)
    index = mbox[_cursor]->cursor();

  printf("Okwayle, on devrait sauver le message #%d\n", index);
}


// print a message
void Application:: print_message(int index)
{
  if (index == CURRENT)
    index = mbox[_cursor]->cursor();

  printf("Okwayle, on devrait printer le message #%d\n", index);
}



// install timer callback in charge of
// checking mail files from time to time
void Application:: set_checkfiles_timer(void) 
{
  XtAppAddTimeOut(interface.context, 1000UL * config.check_delay, checkfiles_timer_handler, this);
}


// le handler appele de temps en temps pour checker les fichiers
void Application:: checkfiles_timer_handler(XtPointer client_data, XtIntervalId *Id)
{
  Application * mythis = (Application *) client_data;

  Id = Id;  // foufferie de warnings
  
//   printf("on check ou quoi!\n");
  mythis->check_files();
  mythis->set_checkfiles_timer();  
}


// regarde si un fichier a bouge, et positionne
// l'icon en consequence
void Application:: check_files(void)
{
  int must_scan = 0;
  
  for (int i = 0; !must_scan && i < nb_files; i++)
    must_scan = mbox[i]->need_a_rescan();

  if (must_scan)
    interface.set_full();
  else
    interface.set_empty();

}





