/*
 * config.C   
 *
 * Handles the configuration object which is able 
 * to be written to and read from a file.
 *
 * Vincent Mallet 1997, 1999
 */

// $Id: config.C,v 1.2 1999/04/20 21:20:16 vmallet Exp $

/* ViMmail - a small mail reader
 * Copyright (C) 1997, 1999 Vincent Mallet - vmallet@enst.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

// I used this file from another software I wrote two years before, 'ZylFree'.
// comments are in french and haven't been updated.


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#define __COMPILING_CONFIG__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <malloc.h>
#include <time.h>

#include <pwd.h>
#include <sys/types.h>
#include <unistd.h>


#include "config.H"


#define stricmp strcasecmp



/*** variables locales ***/

static char *badnum_fmt   = "bad NUMBER parameter: %s\n";   /* vm960705 */
static char *badonoff_fmt = "bad ON/OFF parameter: %s\n";   /* vm960705 */

#define LAST_CHAR(s) ((char *)(s))[strlen(s) - 1]

#define FREE(string) if ((string)) free((string)); (string) = NULL


enum cfgparam  {
  MINCFG=-1,
  TOOLBAR_VISIBLE,
  ASK_BEFORE_QUIT,
  POPUP_ON_STARTUP,
  SHOW_LAST_MESSAGE,
  CHECK_DELAY,
  TEXT_COLOR,
  QUOTE_COLOR,
  HEADER_COLOR,
  HEADER_HIGH_COLOR,
  TAB_EXPAND,
  SPOOL_PATH,
  MAXCFG
}; // enum cfgparam




static char *cfgacronyms[MAXCFG] = {
  "TOOLBAR_VISIBLE",
  "ASK_BEFORE_QUIT",
  "POPUP_ON_STARTUP",
  "SHOW_LAST_MESSAGE",
  "CHECK_DELAY",
  "TEXT_COLOR",
  "QUOTE_COLOR",
  "HEADER_COLOR",
  "HEADER_HIGH_COLOR",
  "TAB_EXPAND",
  "SPOOL_PATH"
};

char *vm_strupr(char *s)
{
  char *p = s;

  while (*p) {
    *p = toupper(*p);
    p++;
  }

  return s;
}



char *rmtrail(char *str)
{
  int i;

  if (0 != (i = strlen(str))) {
    while (--i >= 0) {
      if (!isspace(str[i]))
	break;
    }
    str[++i] = '\0';
  }
  return str;
}

void check_slash( char *path )
{
  char *p;

  if (*path) {
    p = path + strlen(path) - 1;
      if (*p != '/') {
	*++p = '/';
	*++p = '\0';
      }
  }
}




// constructeur
Config:: Config(void)
{
  log_name                   = NULL;
  home_path                  = NULL;
  spool_path                 = NULL;
  initrc_file                = NULL;
  mbox_file                  = NULL;
  mail_file                  = NULL;

  text_color_string          = NULL;
  quote_color_string	     = NULL;
  header_color_string	     = NULL;
  header_high_color_string   = NULL;

  set_default();
}






/*************************
  fonction getconfig
  essaye d'ouvrir le fichier 'ZylFree.cfg' qui devrait se trouver
  dans le path.
  on y recuperera les informations qui nous interesse
**************************/

static int isyesno(char *s)            /* #vm : 1 si s=="YES", 0=="NO", -1==bug */
{
    if (!stricmp(s, "OUI") || !stricmp(s, "YES") || !stricmp(s, "ON") || !stricmp(s, "TRUE") || !stricmp(s, "OUIPS")) return 1;
    if (!stricmp(s, "NON") || !stricmp(s, "NO") || !stricmp(s, "OFF") || !stricmp(s, "FALSE") || !stricmp(s, "NONPS")) return 0;
    return -1;
}

#define CASEYESNO(TAG, var)                                             \
              case TAG:                                                 \
                 if (isyesno(spt2) != -1)                               \
                        var = isyesno(spt2);                            \
                 else                                                   \
                 {                                                      \
                        printf(badonoff_fmt, spt2);     \
                        erreur = 1;                                     \
                 }                                                      \
                 break;

#define CASEINT(TAG, var)                                               \
              case TAG:                                                 \
                 { int i = atoi(spt2);                                  \
                   if (i == 0 && *spt2 != '0')                          \
                   {                                                    \
                        printf(badnum_fmt, spt2);     \
                        erreur = 1;                                     \
                   }                                                    \
                   else                                                 \
                        var = i;                                        \
                 }                                                      \
                 break;

#define CASECHAR(TAG, var)                                              \
              case TAG:                                                 \
                 var = *spt2;                                           \
                 break;

#define CASESTRING(TAG, var)                                            \
              case TAG:                                                 \
	         if (var) free(var);                                    \
                 var = strdup(spt2);                                    \
                 break;

#define CASEPATH(TAG, var)                                              \
              case TAG:                                                 \
                 check_slash(spt2);                                     \
                 if (var) free(var);                                    \
                 var = strdup(spt2);                                    \
                 break;


/* int get_keywordid(char *keyword)
 *
 * retourne l'ID d'un keyword de la config, -1 si keyword inconnu.
 * [cree: 04.09.95 -vm]
 */
static int get_keywordid(char *keyword)
{
    int i, res = -1;

    for (i=0; i<MAXCFG && res == -1; i++) {
      if (0 == strcmp( cfgacronyms[i], keyword ))
        res = i;
    }

    return res;
}



/* int read_configfile( char *filename )
 *
 * lit un fichier de config dans la variable globale "cfg".
 * ne set pas les defaults en entree, ne verifie pas les bornes en sortie.
 *
 * cree pour pouvoir etre appelee plusieurs fois, pour les "#include"
 * retourne TRUE si une erreur quelconque (fichier, option..)
 *          FALSE si tout est OK.
 *
 * cree: 27.09.1995 -vm
 */

int Config:: read_configfile(char *filename)
{
   /*- declarations -*/
   FILE *fin;
   char buf[300], *spt, *spt2;
   int  i, erreur = 0;

   /* si filename ne contient pas de path, rajoute lui le home_path */
   if (strchr(filename, '/'))
     strcpy(buf, filename);
   else {
     strcpy(buf, home_path);
     strcat(buf, filename);
   }

   fin = fopen(buf, "rt"); /* essaye d'ouvrir le fichier */
   if (NULL == fin) { /* si ouverture impossible */
      fprintf(stderr, "Warning: Config:: read_configfile(): unable to open %s\n", buf);
      return TRUE;
   }


   /* #vm: tout la logique pour lire le fichier a ete revue. Au moins, */
   /*      ca marche :-))                                              */
   /*   Scuze pour les accolades, mais quand je comprends plus rien je suis */
   /*   oblige de les aligner, au moins je vois vite celle qui manque :) */
   /*   (sinon je suis sympa, je les mets comme toi :-) */

   while( (spt = fgets( buf, 128, fin )) ) { /* #vm: a fond la caisse, ca craignait :) */  /* sortie sur erreur ou fin de fichier */
                                  /* le but: spt = keyword, spt2 = value */
      LAST_CHAR(spt) = '\0';      /* on vire ce #|[|`\\||'@ de '\n' */
      spt2 = strchr(spt, '#');    /* on commence par virer les commentaires */
      if (spt2) *spt2 = '\0';
      rmtrail(spt);               /* et les blancs en fin de ligne */
      while (*spt == ' ') spt++;  /* et ceux en debut de ligne */
      if (!*spt) continue;        /* ligne vide? => on skip */

      while ((spt2 = strchr(spt, '$')))
	*spt2 = '#';

      spt2 = strchr(spt, '=');    /* on cherche le 'value' */
      if (!spt2) {
          printf("Erreur: ligne ignore (pas de '='): %.40s\n", spt);
          erreur = 1;
          continue;
      }
      *spt2 = '\0';               /* et on coupe la string en deux */
      spt2++;                     /* spt2: pointe sur le 'value' */
      while (*spt2 == ' ') spt2++;
      if (!*spt2) {     /* les lignes qui finissent par '=', ca puir! */
          printf("Erreur: ligne ignore (rien aprs '='): %.38s\n", spt);
          erreur = 1;
          continue;
      }
      rmtrail(spt);
      vm_strupr(spt);                /* on uppercase le tout */

      i = get_keywordid(spt);     /* #vm0409: ahh, enfin :-) */

      if (i>=0)
      {
          switch(i)
          {
	    CASEYESNO(  TOOLBAR_VISIBLE,   toolbar_visible )
	    CASEYESNO(  ASK_BEFORE_QUIT,   ask_before_quit )
	    CASEYESNO(  POPUP_ON_STARTUP,  popup_on_startup )
	    CASEYESNO(  SHOW_LAST_MESSAGE, show_last_message )
	    CASESTRING( TEXT_COLOR,        text_color_string )
	    CASESTRING( QUOTE_COLOR,       quote_color_string )
	    CASESTRING( HEADER_COLOR,      header_color_string )
	    CASESTRING( HEADER_HIGH_COLOR, header_high_color_string )
	    CASEINT(    TAB_EXPAND,        tab_expand )
            CASEINT(    CHECK_DELAY,       check_delay )
	    CASEPATH(   SPOOL_PATH,        spool_path )
	    //              CASEINT( DEFAULT_MSG_SORT, default_msg_sort) /* vm960211*/
	    //              CASEINT( MAX_PAQUETS, max_paquets );           /* #vm0923 */
	    //              CASEPATH( DWL_DIR, dwl_dir );                  /* #vm0909 */
	    //              CASEPATH( UPL_DIR, upl_dir );                  /* #vm0909 */
	    //              CASEPATH( SAV_DIR, sav_dir );                  /* #vm0909 */
/**  //voirapres CASEPATH( WRK_DIR, wrk_dir );  **/          /* #vm0909 */
	    //              CASESTRING( ZIPPER, zipper );                  /* #vm0909 */
	    //              CASESTRING( UNZIPPER, unzipper );              /* #vm0909 */
	    //              CASESTRING( COMSPEC, comspec );                /* #vm0909 */
	    //              CASESTRING( EDITOR, editor );                  /* #vm0909 */
	    //              CASESTRING( TAG_FIC, tag_fic );                /* #vm0909 */
	    //              CASECHAR( READ_CHAR, read_char );              /*  vm0925 */
	    //              CASECHAR( REPLIED_CHAR, replied_char );        /*  vm0925 */
	    //              case WRK_DIR:
//vm0927           strcpy(wbuf, spt2);                           /* #vm0914 */
	    //                   strcpy(wrk_dir, spt2);
	    //                   break;
	    //              case INCLUDE:
	    //                   printf("  [] Inclusion du fichier: %s\n", spt2);
	    //                   if (read_configfile( spt2 ))
	    //                        erreur = TRUE;
	    //                   break;

	  default:
	    fprintf(stderr, "Warning: Config:: read_configfile(): internal tag error: %s\n", spt);
	    break;

              /* autres cas a rajouter */
          }
      }
      else {
	fprintf(stderr, "Warning: unrecognized keyword: %s\n", spt);
	erreur = 1;
      }
    } /* end while */
   fclose (fin);

   return erreur;
}





int Config:: getconfig(void)
{
  char tmp[300];
  int  erreur = FALSE;

//   printf("[Chargement du fichier de configuration... %.35s]\n", initrc_file);

  set_default();

  if (read_configfile(initrc_file))
    erreur = TRUE;

  FREE(mail_file);
  strcat(strcpy(tmp, spool_path), log_name);
  mail_file = strdup(tmp);

/*
  if (erreur)
    {
      fprintf(stderr, "\nPeut-tre bien une touche pour continuer? ");
      //     ext_getch();
      fprintf(stderr, "\n");
    }
*/
  return( EXIT_SUCCESS );
}


void Config:: set_default(void)
{
  char tmp[300];
  char *p;
  uid_t uid;
  struct passwd * pass;

  FREE(log_name);
  FREE(home_path);
  FREE(spool_path);
  FREE(initrc_file);
  FREE(mbox_file);
  FREE(mail_file);

  FREE(text_color_string);
  FREE(header_color_string);
  FREE(header_high_color_string);
  FREE(quote_color_string);


  // recuperation du login name
  uid  = getuid();
  pass = getpwuid(uid);
  log_name = strdup(pass->pw_name);

  // recuperation du home dir
  p = getenv("HOME");
  strcpy(tmp, p);
  check_slash(tmp);
  home_path = strdup(tmp);

  spool_path = strdup("/var/spool/mail/");

  strcat(strcpy(tmp, home_path), ".vmrc");
  initrc_file = strdup(tmp);

  strcat(strcpy(tmp, home_path), "mbox");
  mbox_file = strdup(tmp);

  strcat(strcpy(tmp, spool_path), log_name);
  mail_file = strdup(tmp);


  toolbar_visible          = TRUE;
  ask_before_quit          = TRUE;
  popup_on_startup         = TRUE;
  show_last_message        = TRUE;

  check_delay              = 5;   // check every 15 secs

  text_color_string        = strdup("Black");
  header_color_string      = strdup("DarkBlue");
  header_high_color_string = strdup("Red");
  quote_color_string       = strdup("#A010A0");

  tab_expand               = 4;
}



#define WRITE_BOOL(file,item,var,comment)                           \
            fprintf(file, "%-25s = %-10s  # %s\n",                  \
		   capitalize(cfgacronyms[item]), (var) ? "Yes" : "No", comment);

#define WRITE_STRING(file,item,var,comment)               {         \
	      char buf[200], *p;                                    \
	      strcpy(buf, var);                                     \
	      while ((p = strchr(buf, '#')))                        \
			*p = '$';                                   \
            fprintf(file, "%-25s = %-10s  # %s\n",                  \
		    capitalize(cfgacronyms[item]), buf, comment); }

#define WRITE_INT(file,item,var,comment)                            \
            fprintf(file, "%-25s = %-10d  # %s\n",                  \
		   capitalize(cfgacronyms[item]), var, comment);

#define PRINT(file,string) \
            fprintf(file, "# %s\n", string)

#define NEWLINE(file) \
            fprintf(file, "\n");


int Config:: write_configfile(char *filename)
{
   FILE *fout;
   char buf[300];

   /* si filename ne contient pas de path, rajoute lui le home_path */
   if (strchr(filename, '/'))
     strcpy(buf, filename);
   else {
     strcpy(buf, home_path);
     strcat(buf, filename);
   }

   fout = fopen(buf, "wt"); /* essaye d'ouvrir le fichier */
   if (NULL == fout) { /* si ouverture impossible */
      fprintf(stderr, "Warning: Config:: write_configfile(): unable to open %s\n", buf);
      return TRUE;
   }

   PRINT(fout, "Simple .vmrc file, automatically generated by ViMmail");
   PRINT(fout, "");
   PRINT(fout, "ViMmail has been written by Vincent Mallet (vmallet@enst.fr)");
   PRINT(fout, "");
   PRINT(fout, "Note: next time you save the configuration all the comments will be lost!\n");
   WRITE_STRING(fout, SPOOL_PATH, spool_path, "mail spool path (not the filename)");
   NEWLINE(fout);
   WRITE_BOOL(fout, ASK_BEFORE_QUIT, ask_before_quit, "Ask before quitting ViMmail");
   WRITE_BOOL(fout, TOOLBAR_VISIBLE, toolbar_visible, "Try 'no' if you don't want the toolbar");
   WRITE_BOOL(fout, POPUP_ON_STARTUP, popup_on_startup, "pop up work window on startup");
   WRITE_BOOL(fout, SHOW_LAST_MESSAGE, show_last_message, "show last message in mailbox (otherwise, first one");
   NEWLINE(fout);
   PRINT(fout, "Color Definition.   !! WARNING !!  use \"$\" instead of \"#\" in color names!");
   WRITE_STRING(fout, HEADER_COLOR, header_color_string, "Message header color");
   WRITE_STRING(fout, HEADER_HIGH_COLOR, header_high_color_string, "Message header color when highlighted");
   WRITE_STRING(fout, QUOTE_COLOR, quote_color_string, "Quoted text color");
   WRITE_STRING(fout, TEXT_COLOR, text_color_string, "Message text color");
   NEWLINE(fout);
   WRITE_INT(fout, TAB_EXPAND, tab_expand, "Number of spaces for one tab");
   WRITE_INT(fout, CHECK_DELAY, check_delay, "Number of seconds between checks (0 for none)");

   fclose(fout);

   return FALSE;
}


void Config:: save_config(void)
{
  write_configfile(initrc_file);
}







char * Config:: capitalize(char *s)
{
  static char buf[50], *p;
  int last_was_space = TRUE;

  p = buf;
  while (*s) {
    if (last_was_space)
      *p++ = toupper(*s);
    else
      *p++ = tolower(*s);
    last_was_space = (*s++ == '_');
  }
  *p = '\0';

  return buf;
}



/* fin de Config.c */

