/*
 * motifutil.C
 *
 * Some Motif helper functions.
 *
 * Vincent Mallet 1997, 1999
 */

// $Id: motifutil.C,v 1.2 1999/04/20 21:46:06 vmallet Exp $

/* ViMmail - a small mail reader
 * Copyright (C) 1997, 1999 Vincent Mallet - vmallet@enst.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdlib.h>

#include <Xm/Xm.h>
#include <Xm/MessageB.h>
#include <Xm/SashP.h>         // pas le choix

#include "vimmail.H"
#include "application.H"
#include "motifutil.H"



// remove a button from a dialog
void remove_button(Widget dialog, int button)
     /* Removes button number `btn' from dialog `dlg'. */
{
  Widget b;

  b = XmMessageBoxGetChild(dialog, button);
  XtUnmanageChild(b);
} // remove_button




// Avoid the sash traversal in a paned window
void TurnOffSashTraversal(Widget panedw)
{
  Widget *children;
  int num_children;

  XtVaGetValues(panedw, XmNchildren, &children,
		        XmNnumChildren, &num_children,
		        NULL);

  while (num_children--)
    if (XmIsSash(children[num_children]))
      XtVaSetValues(children[num_children], XmNtraversalOn, False, NULL);
}


// change cursor shape for the specified widget
void nice_cursor(Widget widget, int shape)
{
  Cursor cursor;
  Display *dpy;
  Window  w;

  dpy = XtDisplay(widget);
  w   = XtWindow(widget);

  cursor = XCreateFontCursor(dpy, shape);
  XDefineCursor(dpy, w, cursor);
  XFreeCursor(dpy, cursor);
}




static void quit_handler(Widget w, XtPointer client_data, XtPointer cbs)
{
  w = w; client_data = cbs;
  vimmail.end_app();
}


// quit the application
void quit(void)
{
  static Widget dialog;

  if (vimmail.config.ask_before_quit) {
    if (!dialog) {
      dialog = XmCreateWarningDialog(vimmail.interface.main_window, "QuitDialog", NULL, 0);
      XtAddCallback(dialog, XmNokCallback, quit_handler, NULL);
      remove_button(dialog, XmDIALOG_HELP_BUTTON);
      XtManageChild(dialog);
    }
    else
      XtManageChild(dialog);
  }
  else
    vimmail.end_app();
}


// retrieve parent shell
Widget GetTopShell(Widget w)
{
  while (w && !XtIsWMShell(w))
    w = XtParent(w);

  return w;
}



