/*
 * toolbar.C
 *
 * The Tool Bar.
 *
 * Vincent Mallet 1997, 1999
 */

// $Id: toolbar.C,v 1.2 1999/04/20 22:01:20 vmallet Exp $

/* ViMmail - a small mail reader
 * Copyright (C) 1997, 1999 Vincent Mallet - vmallet@enst.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include <Xm/Xm.h>
#include <Xm/Frame.h>
#include <Xm/PushB.h>
#include <Xm/Form.h>

#include "vimmail.H"
#include "application.H"
#include "fileselect.H"
#include "toolbar.H"
#include "motifutil.H"

#include "pushbuttons.h"
#include "cluepushbutton.h"

#include "Pixmaps/print.xpm"
#include "Pixmaps/save2.xpm"
#include "Pixmaps/open.xpm"
#include "Pixmaps/close.xpm"
#include "Pixmaps/cut.xpm"
#include "Pixmaps/copy.xpm"
#include "Pixmaps/paste.xpm"
#include "Pixmaps/help.xpm"
#include "Pixmaps/popdown.xpm"
#include "Pixmaps/quit.xpm"
#include "Pixmaps/rescan.xpm"
#include "Pixmaps/rescan_all.xpm"
#include "Pixmaps/reply.xpm"

enum { /* Buttons IDs */
  ButtonNone,
  ButtonOpen,
  ButtonRescan,
  ButtonRescanAll,
  ButtonReply,
  ButtonClose,
  ButtonSave,
  ButtonPrint,
  ButtonAbout,
  ButtonQuit,
  ButtonPopdown
};



// Actually create this toolbar
void ToolBar:: make(Widget parent)
{
  Widget button, form;

  widget = XtVaCreateManagedWidget("ToolFrame", xmFrameWidgetClass, parent, NULL);

  form = XmCreateForm(widget, "ToolBarForm", NULL, 0);


  // open, rescan, rescan all, close
  button = XmCreateCluePushButton(form, "OpenButton", "Open a mail file", NULL, 0);
  CreatePixmapButton(button, open_xpm, open_xpm, NULL);
  XtAddCallback(button, XmNactivateCallback, button_handler, (XtPointer) ButtonOpen);
  XtManageChild(button);

  button = XmCreateCluePushButton(form, "RescanButton", "Rescan current mail file", NULL, 0);
  CreatePixmapButton(button, rescan_xpm, rescan_xpm, NULL);
  XtAddCallback(button, XmNactivateCallback, button_handler, (XtPointer) ButtonRescan);
  XtManageChild(button);

  button = XmCreateCluePushButton(form, "RescanAllButton", "Rescan all mail files", NULL, 0);
  CreatePixmapButton(button, rescan_all_xpm, rescan_all_xpm, NULL);
  XtAddCallback(button, XmNactivateCallback, button_handler, (XtPointer) ButtonRescanAll);
  XtManageChild(button);

  button = XmCreateCluePushButton(form, "CloseButton","Close current mail file", NULL, 0);
  CreatePixmapButton(button, close_xpm, close_xpm, NULL);
  //  XtAddCallback(button, XmNactivateCallback, open_handler, NULL);
  XtManageChild(button);

  // reply, save, print
  button = XmCreateCluePushButton(form, "ReplyButton", "Reply to current message", NULL, 0);
  CreatePixmapButton(button, reply_xpm, reply_xpm, NULL);
  XtAddCallback(button, XmNactivateCallback, button_handler, (XtPointer) ButtonReply);
  XtManageChild(button);

  button = XmCreateCluePushButton(form, "SaveButton", "Save current message", NULL, 0);
  CreatePixmapButton(button, save2_xpm, save2_xpm, NULL);
  XtAddCallback(button, XmNactivateCallback, button_handler, (XtPointer) ButtonSave);
  XtManageChild(button);

  button = XmCreateCluePushButton(form, "PrintButton", "Print current message", NULL, 0);
  CreatePixmapButton(button, print_xpm, print_xpm, NULL);
  XtAddCallback(button, XmNactivateCallback, button_handler, (XtPointer) ButtonPrint);
  XtManageChild(button);

  // cut, copy, paste
  button = XmCreateCluePushButton(form, "CutButton", "Cut", NULL, 0);
  CreatePixmapButton(button, cut_xpm, cut_xpm, NULL);
  //  XtAddCallback(button, XmNactivateCallback, quit_handler, NULL);
  XtManageChild(button);

  button = XmCreateCluePushButton(form, "CopyButton", "Copy", NULL, 0);
  CreatePixmapButton(button, copy_xpm, copy_xpm, NULL);
  //  XtAddCallback(button, XmNactivateCallback, quit_handler, NULL);
  XtManageChild(button);

  button = XmCreateCluePushButton(form, "PasteButton", "Paste", NULL, 0);
  CreatePixmapButton(button, paste_xpm, paste_xpm, NULL);
  //  XtAddCallback(button, XmNactivateCallback, quit_handler, NULL);
  XtManageChild(button);

  // about
  button = XmCreateCluePushButton(form, "AboutButton", "About ViMmail", NULL, 0);
  CreatePixmapButton(button, help_xpm, help_xpm, NULL);
  XtAddCallback(button, XmNactivateCallback, button_handler, (XtPointer) ButtonAbout);
  XtManageChild(button);

  // quit
  button = XmCreateCluePushButton(form, "QuitButton", "Quit ViMmail", NULL, 0);
  CreatePixmapButton(button, quit_xpm, quit_xpm, NULL);
  XtAddCallback(button, XmNactivateCallback, quit_handler, NULL);
  XtManageChild(button);

  // popdown
  button = XmCreateCluePushButton(form, "PopButton", "Popdown Workwindow", NULL, 0);
  CreatePixmapButton(button, popdown_xpm, popdown_xpm, NULL);
  XtAddCallback(button, XmNactivateCallback, button_handler, (XtPointer) ButtonPopdown);
  XtManageChild(button);

  XtManageChild(form);
}


void ToolBar:: button_handler(Widget w, XtPointer client_data, XtPointer cbs)
{
  w = w; cbs = cbs; 

  switch((int) client_data) {
  case ButtonOpen:
    vimmail.interface.fileselect.show_dialog();
    break;
  case ButtonRescan:
    vimmail.rescan_message_file();
    break;
  case ButtonRescanAll:
    vimmail.rescan_all_message_files();
    break;
  case ButtonAbout:
    vimmail.interface.about.show_dialog();
    break;
  case ButtonPopdown:
    vimmail.interface.popdown();
    break;
  case ButtonSave:
    vimmail.interface.set_full();
    vimmail.save_message();
    break;
  case ButtonPrint:
    vimmail.print_message();
    break;
  }
}

void ToolBar:: quit_handler(Widget w, XtPointer client_data, XtPointer cbs)
{
  w = w; cbs = cbs; client_data = client_data;
  
  quit();
}


// Set this toolbar visible or not
void ToolBar:: set_visible(int visible)
{
  if (visible)
    XtManageChild(widget);
  else
    XtUnmanageChild(widget);
}
